@@ -1,4 +1,4 @@
-/* $Id: ruby_xml_node.c 497 2008-07-22 01:33:51Z cfis $ */
+/* $Id: ruby_xml_node.c 612 2008-11-21 08:01:29Z cfis $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
@@ -6,9 +6,17 @@
 #include "ruby_xml_node.h"
 
 VALUE cXMLNode;
-VALUE eXMLNodeSetNamespace;
-VALUE eXMLNodeFailedModify;
-VALUE eXMLNodeUnknownType;
+
+/* Document-class: LibXML::XML::Node
+ *
+ * Nodes are the primary objects that make up an XML document. 
+ * The node class represents most node types that are found in
+ * an XML document (but not Attributes, see LibXML::XML::Attribute).
+ * It exposes libxml's full API for creating, querying
+ * moving and deleting node objects.  Many of these methods are
+ * documented in the DOM Level 3 specification found at:
+ * http://www.w3.org/TR/DOM-Level-3-Core/. */
+
 
 VALUE
 check_string_or_symbol( VALUE val ) {
@@ -26,8 +34,8 @@ check_string_or_symbol( VALUE val ) {
  * 
  * Obtain this node's base URI.
  */
-VALUE
-ruby_xml_node_base_get(VALUE self) {
+static VALUE
+rxml_node_base_get(VALUE self) {
   xmlNodePtr xnode;
   xmlChar* base_uri;            
   VALUE result = Qnil;
@@ -55,8 +63,8 @@ ruby_xml_node_base_get(VALUE self) {
  * 
  * Set this node's base URI.
  */
-VALUE
-ruby_xml_node_base_set(VALUE self, VALUE uri) {
+static VALUE
+rxml_node_base_set(VALUE self, VALUE uri) {
   xmlNodePtr xnode;
 
   Check_Type(uri, T_STRING);
@@ -75,8 +83,8 @@ ruby_xml_node_base_set(VALUE self, VALUE uri) {
  * 
  * Obtain this node's content as a string.
  */
-VALUE
-ruby_xml_node_content_get(VALUE self) {
+static VALUE
+rxml_node_content_get(VALUE self) {
   xmlNodePtr xnode;
   xmlChar *content;
   VALUE result = Qnil;
@@ -97,8 +105,8 @@ ruby_xml_node_content_get(VALUE self) {
  * 
  * Set this node's content to the specified string.
  */ 
-VALUE
-ruby_xml_node_content_set(VALUE self, VALUE content) {
+static VALUE
+rxml_node_content_set(VALUE self, VALUE content) {
   xmlNodePtr xnode;
 
   Check_Type(content, T_STRING);
@@ -118,8 +126,8 @@ ruby_xml_node_content_set(VALUE self, VALUE content) {
  * *Deprecated*: Stripped content can be obtained via the
  * +content+ method.
  */
-VALUE
-ruby_xml_node_content_stripped_get(VALUE self) {
+static VALUE
+rxml_node_content_stripped_get(VALUE self) {
   xmlNodePtr xnode;
   xmlChar* content;
   VALUE result = Qnil;
@@ -143,14 +151,14 @@ ruby_xml_node_content_stripped_get(VALUE self) {
  * 
  * Returns this node's first child node if any.
  */
-VALUE
-ruby_xml_node_first_get(VALUE self) {
+static VALUE
+rxml_node_first_get(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
 
   if (xnode->children)
-    return(ruby_xml_node2_wrap(cXMLNode, xnode->children));
+    return(rxml_node2_wrap(cXMLNode, xnode->children));
   else
     return(Qnil);
 }
@@ -160,8 +168,8 @@ ruby_xml_node_first_get(VALUE self) {
  * underlying for child_set and child_add, difference being
  * former raises on implicit copy, latter does not.
  */
-VALUE
-ruby_xml_node_child_set_aux(VALUE self, VALUE rnode) {
+static VALUE
+rxml_node_child_set_aux(VALUE self, VALUE rnode) {
   xmlNodePtr pnode, chld, ret;
 
   if (rb_obj_is_kind_of(rnode, cXMLNode) == Qfalse)
@@ -175,14 +183,14 @@ ruby_xml_node_child_set_aux(VALUE self, VALUE rnode) {
 
   ret = xmlAddChild(pnode, chld);
   if (ret == NULL) {
-    rb_raise(eXMLNodeFailedModify, "unable to add a child to the document");
+    rxml_raise(&xmlLastError);
   } else if ( ret==chld ) {
     /* child was added whole to parent and we need to return it as a new object */
-    return ruby_xml_node2_wrap(cXMLNode,chld);
+    return rxml_node2_wrap(cXMLNode,chld);
   }
   /* else */
   /* If it was a text node, then ret should be parent->last, so we will just return ret. */
-  return ruby_xml_node2_wrap(cXMLNode,ret);
+  return rxml_node2_wrap(cXMLNode,ret);
 }
 
 /*
@@ -191,9 +199,9 @@ ruby_xml_node_child_set_aux(VALUE self, VALUE rnode) {
  * 
  * Set a child node for this node. Also called for <<
  */
-VALUE
-ruby_xml_node_child_set(VALUE self, VALUE rnode) {
-  return ruby_xml_node_child_set_aux(self, rnode);
+static VALUE
+rxml_node_child_set(VALUE self, VALUE rnode) {
+  return rxml_node_child_set_aux(self, rnode);
 }
 
 
@@ -202,10 +210,12 @@ ruby_xml_node_child_set(VALUE self, VALUE rnode) {
  *    node << ("string" | node) -> XML::Node
  * 
  * Add the specified string or XML::Node to this node's
- * content.
+ * content.  The returned node is the node that was
+ * added and not self, thereby allowing << calls to
+ * be chained.
  */
-VALUE
-ruby_xml_node_content_add(VALUE self, VALUE obj) {
+static VALUE
+rxml_node_content_add(VALUE self, VALUE obj) {
   xmlNodePtr xnode;
   VALUE str;
 
@@ -215,19 +225,15 @@ ruby_xml_node_content_add(VALUE self, VALUE obj) {
    * danj 070827
    */
   if (rb_obj_is_kind_of(obj, cXMLNode)) {
-    ruby_xml_node_child_set(self, obj);
-    return(self);
-  } else if (TYPE(obj) == T_STRING) {
-    xmlNodeAddContent(xnode, (xmlChar*)StringValuePtr(obj));
-    return(self);
+    rxml_node_child_set(self, obj);
   } else {
     str = rb_obj_as_string(obj);
     if (NIL_P(str) || TYPE(str) != T_STRING)
       rb_raise(rb_eTypeError, "invalid argument: must be string or XML::Node");
 
-    xmlNodeAddContent(xnode, (xmlChar*)StringValuePtr(str));
-    return(self);
+		xmlNodeAddContent(xnode, (xmlChar*)StringValuePtr(str));
   }
+  return(self);
 }
 
 /*
@@ -236,9 +242,9 @@ ruby_xml_node_content_add(VALUE self, VALUE obj) {
  * 
  * Set a child node for this node.
  */
-VALUE
-ruby_xml_node_child_add(VALUE self, VALUE rnode) {
-  return ruby_xml_node_child_set_aux(self, rnode);
+static VALUE
+rxml_node_child_add(VALUE self, VALUE rnode) {
+  return rxml_node_child_set_aux(self, rnode);
 }
 
 /*
@@ -247,8 +253,8 @@ ruby_xml_node_child_add(VALUE self, VALUE rnode) {
  * 
  * Obtain the XML::Document this node belongs to.
  */
-VALUE
-ruby_xml_node_doc(VALUE self) {
+static VALUE
+rxml_node_doc(VALUE self) {
   xmlNodePtr xnode;
   xmlDocPtr doc=NULL;
 
@@ -292,8 +298,8 @@ ruby_xml_node_doc(VALUE self) {
  * 
  * Dump this node to stdout.
  */
-VALUE
-ruby_xml_node_dump(VALUE self) {
+static VALUE
+rxml_node_dump(VALUE self) {
   xmlNodePtr xnode;
   xmlBufferPtr buf;
 
@@ -317,8 +323,8 @@ ruby_xml_node_dump(VALUE self) {
  * Dump this node to stdout, including any debugging
  * information.
  */
-VALUE
-ruby_xml_node_debug_dump(VALUE self) {
+static VALUE
+rxml_node_debug_dump(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
 
@@ -340,8 +346,8 @@ ruby_xml_node_debug_dump(VALUE self) {
  *  doc = XML::Document.new('model/books.xml')
  *  doc.root.each {|node| puts node}
  */
-VALUE
-ruby_xml_node_each(VALUE self) {
+static VALUE
+rxml_node_each(VALUE self) {
   xmlNodePtr xnode;
   xmlNodePtr xchild;
   Data_Get_Struct(self, xmlNode, xnode);
@@ -349,7 +355,7 @@ ruby_xml_node_each(VALUE self) {
   xchild = xnode->children;
   
   while (xchild) {
-    rb_yield(ruby_xml_node2_wrap(cXMLNode, xchild));
+    rb_yield(rxml_node2_wrap(cXMLNode, xchild));
     xchild = xchild->next;
   }
   return Qnil;
@@ -362,8 +368,8 @@ ruby_xml_node_each(VALUE self) {
  * 
  * Determine whether this node is empty.
  */
-VALUE
-ruby_xml_node_empty_q(VALUE self) {
+static VALUE
+rxml_node_empty_q(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
   if (xnode == NULL)
@@ -373,7 +379,7 @@ ruby_xml_node_empty_q(VALUE self) {
 }
 
 
-VALUE ruby_xml_node_to_s(VALUE self);
+static VALUE rxml_node_to_s(VALUE self);
 
 /*
  * call-seq:
@@ -381,8 +387,8 @@ VALUE ruby_xml_node_to_s(VALUE self);
  * 
  * Test equality between the two nodes. Two nodes are equal
  * if they are the same node or have the same XML representation.*/
-VALUE
-ruby_xml_node_eql_q(VALUE self, VALUE other) {
+static VALUE
+rxml_node_eql_q(VALUE self, VALUE other) {
   if (self == other)
   {
     return Qtrue;
@@ -399,8 +405,8 @@ ruby_xml_node_eql_q(VALUE self, VALUE other) {
     if (rb_obj_is_kind_of(other, cXMLNode) == Qfalse)
       rb_raise(rb_eTypeError, "Nodes can only be compared against other nodes");
     
-    self_xml = ruby_xml_node_to_s(self);
-    other_xml = ruby_xml_node_to_s(other);
+    self_xml = rxml_node_to_s(self);
+    other_xml = rxml_node_to_s(other);
     return(rb_funcall(self_xml, rb_intern("=="), 1, other_xml));
   }    
 }
@@ -412,8 +418,8 @@ ruby_xml_node_eql_q(VALUE self, VALUE other) {
  * Create a new #CDATA node, optionally setting
  * the node's content.
  */
-VALUE
-ruby_xml_node_new_cdata(int argc, VALUE *argv, VALUE class) {
+static VALUE
+rxml_node_new_cdata(int argc, VALUE *argv, VALUE class) {
   xmlNodePtr xnode;
   VALUE str=Qnil;
 
@@ -430,7 +436,7 @@ ruby_xml_node_new_cdata(int argc, VALUE *argv, VALUE class) {
     if (xnode == NULL)
       return(Qnil);
       
-    return ruby_xml_node2_wrap(class,xnode);
+    return rxml_node2_wrap(class,xnode);
 
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (1)");
@@ -449,8 +455,8 @@ ruby_xml_node_new_cdata(int argc, VALUE *argv, VALUE class) {
  * the node's content.
  * 
  */
-VALUE
-ruby_xml_node_new_comment(int argc, VALUE *argv, VALUE class) {
+static VALUE
+rxml_node_new_comment(int argc, VALUE *argv, VALUE class) {
   xmlNodePtr xnode;
   VALUE str=Qnil;
 
@@ -468,7 +474,7 @@ ruby_xml_node_new_comment(int argc, VALUE *argv, VALUE class) {
     if (xnode == NULL)
       return(Qnil);
 
-    return ruby_xml_node2_wrap(class,xnode);
+    return rxml_node2_wrap(class,xnode);
 
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (1)");
@@ -486,8 +492,8 @@ ruby_xml_node_new_comment(int argc, VALUE *argv, VALUE class) {
  * Obtain the language set for this node, if any.
  * This is set in XML via the xml:lang attribute.
  */
-VALUE
-ruby_xml_node_lang_get(VALUE self) {
+static VALUE
+rxml_node_lang_get(VALUE self) {
   xmlNodePtr xnode;
   xmlChar *lang;
   VALUE result = Qnil;
@@ -513,8 +519,8 @@ ruby_xml_node_lang_get(VALUE self) {
  * Set the language for this node. This affects the value
  * of the xml:lang attribute.
  */
-VALUE
-ruby_xml_node_lang_set(VALUE self, VALUE lang) {
+static VALUE
+rxml_node_lang_set(VALUE self, VALUE lang) {
   xmlNodePtr xnode;
 
   Check_Type(lang, T_STRING);
@@ -531,14 +537,14 @@ ruby_xml_node_lang_set(VALUE self, VALUE lang) {
  * 
  * Obtain the last child node of this node, if any.
  */
-VALUE
-ruby_xml_node_last_get(VALUE self) {
+static VALUE
+rxml_node_last_get(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
 
   if (xnode->last)
-    return(ruby_xml_node2_wrap(cXMLNode, xnode->last));
+    return(rxml_node2_wrap(cXMLNode, xnode->last));
   else
     return(Qnil);
 }
@@ -551,8 +557,8 @@ ruby_xml_node_last_get(VALUE self) {
  * node was read from. If +default_line_numbers+ is set
  * false (the default), this method returns zero.
  */
-VALUE
-ruby_xml_node_line_num(VALUE self) {
+static VALUE
+rxml_node_line_num(VALUE self) {
   xmlNodePtr xnode;
   long line_num;
   Data_Get_Struct(self, xmlNode, xnode);
@@ -574,8 +580,8 @@ ruby_xml_node_line_num(VALUE self) {
  * 
  * Determine whether this node is an xlink node.
  */
-VALUE
-ruby_xml_node_xlink_q(VALUE self) {
+static VALUE
+rxml_node_xlink_q(VALUE self) {
   xmlNodePtr xnode;
   xlinkType xlt;
 
@@ -597,8 +603,8 @@ ruby_xml_node_xlink_q(VALUE self) {
  * If this is not an xlink node (see +xlink?+), will return
  * nil.
  */
-VALUE
-ruby_xml_node_xlink_type(VALUE self) {
+static VALUE
+rxml_node_xlink_type(VALUE self) {
   xmlNodePtr xnode;
   xlinkType xlt;
 
@@ -620,8 +626,8 @@ ruby_xml_node_xlink_type(VALUE self) {
  * If this is not an xlink node (see +xlink?+), will return
  * nil.
  */
-VALUE
-ruby_xml_node_xlink_type_name(VALUE self) {
+static VALUE
+rxml_node_xlink_type_name(VALUE self) {
   xmlNodePtr xnode;
   xlinkType xlt;
 
@@ -648,8 +654,8 @@ ruby_xml_node_xlink_type_name(VALUE self) {
  * 
  * Obtain this node's name.
  */
-VALUE
-ruby_xml_node_name_get(VALUE self) {
+static VALUE
+rxml_node_name_get(VALUE self) {
   xmlNodePtr xnode;
   const xmlChar *name;
 
@@ -696,8 +702,8 @@ ruby_xml_node_name_get(VALUE self) {
  * 
  * Set this node's name.
  */
-VALUE
-ruby_xml_node_name_set(VALUE self, VALUE name) {
+static VALUE
+rxml_node_name_set(VALUE self, VALUE name) {
   xmlNodePtr xnode;
 
   Check_Type(name, T_STRING);
@@ -714,8 +720,8 @@ ruby_xml_node_name_set(VALUE self, VALUE name) {
  * Obtain an array of +XML::NS+ objects representing
  * this node's xmlns attributes
  */
-VALUE
-ruby_xml_node_namespace_get(VALUE self) {
+static VALUE
+rxml_node_namespace_get(VALUE self) {
   xmlNodePtr xnode;
   xmlNsPtr *nsList, *cur;
   VALUE arr, ns;
@@ -731,7 +737,7 @@ ruby_xml_node_namespace_get(VALUE self) {
 
   arr = rb_ary_new();
   for (cur = nsList; *cur != NULL; cur++) {
-    ns = ruby_xml_ns_wrap(*cur);
+    ns = rxml_ns_wrap(*cur);
     if (ns == Qnil)
       continue;
     else
@@ -749,15 +755,15 @@ ruby_xml_node_namespace_get(VALUE self) {
  * 
  * Obtain this node's namespace node.
  */
-VALUE
-ruby_xml_node_namespace_get_node(VALUE self) {
+static VALUE
+rxml_node_namespace_get_node(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
   if (xnode->ns == NULL)
     return(Qnil);
   else
-    return ruby_xml_ns_wrap(xnode->ns);
+    return rxml_ns_wrap(xnode->ns);
 }
 
 // TODO namespace_set can take varargs (in fact, must if used
@@ -773,8 +779,8 @@ ruby_xml_node_namespace_get_node(VALUE self) {
  * 
  * Add the specified XML::NS object to this node's xmlns attributes.
  */
-VALUE
-ruby_xml_node_namespace_set(int argc, VALUE *argv, VALUE self) {
+static VALUE
+rxml_node_namespace_set(int argc, VALUE *argv, VALUE self) {
   VALUE rns, rprefix;
   xmlNodePtr xnode;
   xmlNsPtr xns;
@@ -812,9 +818,9 @@ ruby_xml_node_namespace_set(int argc, VALUE *argv, VALUE self) {
 
     xns = xmlNewNs(xnode, (xmlChar*)href, (xmlChar*)StringValuePtr(rprefix));
     if (xns == NULL)
-      rb_raise(eXMLNodeSetNamespace, "unable to set the namespace");
+      rxml_raise(&xmlLastError);
     else
-      return ruby_xml_ns_wrap(xns);
+      return rxml_ns_wrap(xns);
     break;
 
   default:
@@ -838,22 +844,19 @@ ruby_xml_node_namespace_set(int argc, VALUE *argv, VALUE self) {
  * If the xmlNode has no parent or document, then call xmlFree.
  */
 void
-ruby_xml_node2_free(xmlNodePtr xnode) {
+rxml_node2_free(xmlNodePtr xnode) {
 
   if (xnode != NULL) {
     xnode->_private=NULL;
 
     if (xnode->doc==NULL && xnode->parent==NULL) {
-#ifdef NODE_DEBUG
-      fprintf(stderr,"ruby_xml_node2_free ruby_xfree rxn=0x%x xn=0x%x o=0x%x\n",(long)rxn,(long)xnode,(long)xnode->_private);
-#endif
       xmlFreeNode(xnode);
     }
   }
 }
 
 void
-ruby_xml_node_mark_common(xmlNodePtr xnode) {
+rxml_node_mark_common(xmlNodePtr xnode) {
   if (xnode->parent == NULL ) {
 #ifdef NODE_DEBUG
     fprintf(stderr,"mark no parent r=0x%x *n=0x%x\n",rxn,node);
@@ -871,16 +874,13 @@ ruby_xml_node_mark_common(xmlNodePtr xnode) {
       rb_warning("XmlNode Root Parent is not bound! (%s:%d)",
 		 __FILE__,__LINE__);
     else {
-#ifdef NODE_DEBUG
-      fprintf(stderr,"mark rxn=0x%x xn=0x%x o=0x%x doc=0x%x\n",(long)rxn,(long)node,(long)node->_private,node->doc);
-#endif
       rb_gc_mark((VALUE)xnode->_private);
     }
   }
 }
 
 void
-ruby_xml_node2_mark(xmlNodePtr xnode) {
+rxml_node2_mark(xmlNodePtr xnode) {
   if (xnode == NULL ) return;
 
   if (xnode->_private == NULL ) {
@@ -889,66 +889,61 @@ ruby_xml_node2_mark(xmlNodePtr xnode) {
     return;
   }
 
-  ruby_xml_node_mark_common(xnode);
+  rxml_node_mark_common(xnode);
 }
 
 VALUE
-ruby_xml_node2_wrap(VALUE class, xmlNodePtr xnode)
+rxml_node2_wrap(VALUE class, xmlNodePtr xnode)
 {
   VALUE obj;
 
   // This node is already wrapped
   if (xnode->_private != NULL) {
-#ifdef NODE_DEBUG
-    Data_Get_Struct((VALUE)xnode->_private,ruby_xml_node,rxn);
-    fprintf(stderr,"re-wrap rn=0x%x n*=0x%x\n",(long)rxn,(long)xnode);
-#endif
     return (VALUE)xnode->_private;
   }
 
   obj=Data_Wrap_Struct(class,
-                       ruby_xml_node2_mark, ruby_xml_node2_free,
+                       rxml_node2_mark, rxml_node2_free,
                        xnode);
 
   xnode->_private=(void*)obj;
-#ifdef NODE_DEBUG
-  fprintf(stderr,"wrap rn=0x%x n*=0x%x d*=0x%x\n",
-	  (long)rxn,(long)xnode,xnode->doc);
-#endif
   return obj;
 }
 
-VALUE
-ruby_xml_node2_new_native(VALUE class, VALUE ns, VALUE name)
+/*
+ * call-seq:
+ *    XML::Node.new_string(namespace, name, content) -> XML::Node
+ * 
+ * Create a new element node in the specified namespace with the 
+ * supplied name and content.
+ */
+static VALUE
+rxml_node_new_string(VALUE klass, VALUE ns, VALUE name, VALUE val)
 {
-  VALUE obj;
   xmlNodePtr xnode;
-  xmlNsPtr xns=NULL;
+  xmlNsPtr xns = NULL;
+  VALUE node;
 
-  if ( ! NIL_P(ns) ) {
-    Data_Get_Struct(ns,xmlNs,xns);
+  if (!NIL_P(ns)) {
+    Data_Get_Struct(ns, xmlNs, xns);
   }
-  xnode=xmlNewNode(xns,(xmlChar*)StringValuePtr(name));
+  
+  xnode = xmlNewNode(xns,(xmlChar*)StringValuePtr(name));
   xnode->_private=NULL;
 
-  obj= ruby_xml_node2_wrap(class,xnode);
+  node = rxml_node2_wrap(klass, xnode);
+  rb_obj_call_init(node, 0, NULL);
   
-  rb_obj_call_init(obj,0,NULL);
-  return obj;
+  if (!NIL_P(val))
+  {
+    if (TYPE(val) != T_STRING)
+      val = rb_obj_as_string(val);
+
+    rxml_node_content_set(node, val);
+  }
+  return node;
 }
 
-VALUE
-ruby_xml_node2_new_string(VALUE class, VALUE ns, VALUE name, VALUE val)
-{
-  VALUE obj;
-  obj=ruby_xml_node2_new_native(class,ns,name);
-  if ( ! NIL_P(val) ) {
-    if ( TYPE(val) != T_STRING )
-      val=rb_obj_as_string(val);
-    ruby_xml_node_content_set(obj,val);
-  }
-  return obj;
-}
 /*
  * call-seq:
  *    XML::Node.new(name, content = nil) -> XML::Node
@@ -958,8 +953,8 @@ ruby_xml_node2_new_string(VALUE class, VALUE ns, VALUE name, VALUE val)
  * the node's content.
  * backward compatibility for <.5 new
  */
-VALUE
-ruby_xml_node2_new_string_bc(int argc, VALUE *argv, VALUE class)
+static VALUE
+rxml_node_new_string_bc(int argc, VALUE *argv, VALUE class)
 {
   VALUE content=Qnil;
   VALUE name=Qnil;
@@ -971,7 +966,7 @@ ruby_xml_node2_new_string_bc(int argc, VALUE *argv, VALUE class)
     
   case 1:
     name=check_string_or_symbol( argv[0] );
-    return ruby_xml_node2_new_string(class,Qnil,name,content);
+    return rxml_node_new_string(class,Qnil,name,content);
 
   default:
     rb_raise(rb_eArgError, "wrong number of arguments (1 or 2) given %d",argc);
@@ -984,14 +979,14 @@ ruby_xml_node2_new_string_bc(int argc, VALUE *argv, VALUE class)
  * 
  * Obtain the next sibling node, if any.
  */
-VALUE
-ruby_xml_node_next_get(VALUE self) {
+static VALUE
+rxml_node_next_get(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
 
   if (xnode->next)
-    return(ruby_xml_node2_wrap(cXMLNode, xnode->next));
+    return(rxml_node2_wrap(cXMLNode, xnode->next));
   else
     return(Qnil);
 }
@@ -1003,8 +998,8 @@ ruby_xml_node_next_get(VALUE self) {
  * 
  * Insert the specified node as this node's next sibling.
  */
-VALUE
-ruby_xml_node_next_set(VALUE self, VALUE rnode) {
+static VALUE
+rxml_node_next_set(VALUE self, VALUE rnode) {
   xmlNodePtr cnode, pnode, ret;
 
   if (rb_obj_is_kind_of(rnode, cXMLNode) == Qfalse)
@@ -1015,9 +1010,9 @@ ruby_xml_node_next_set(VALUE self, VALUE rnode) {
 
   ret = xmlAddNextSibling(pnode, cnode);
   if (ret == NULL)
-    rb_raise(eXMLNodeFailedModify, "unable to add a sibling to the document");
+    rxml_raise(&xmlLastError);
 
-  return(ruby_xml_node2_wrap(cXMLNode, ret));
+  return(rxml_node2_wrap(cXMLNode, ret));
 }
 
 
@@ -1027,8 +1022,8 @@ ruby_xml_node_next_set(VALUE self, VALUE rnode) {
  * 
  * Determine whether this node has a namespace.
  */
-VALUE
-ruby_xml_node_ns_q(VALUE self) {
+static VALUE
+rxml_node_ns_q(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
   if (xnode->ns == NULL)
@@ -1044,8 +1039,8 @@ ruby_xml_node_ns_q(VALUE self) {
  * Obtain an array of +XML::NS+ objects representing
  * this node's xmlns attributes
  */
-VALUE
-ruby_xml_node_ns_def_q(VALUE self) {
+static VALUE
+rxml_node_ns_def_q(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
   if (xnode->nsDef == NULL)
@@ -1061,14 +1056,14 @@ ruby_xml_node_ns_def_q(VALUE self) {
  * 
  * Obtain this node's default namespace.
  */
-VALUE
-ruby_xml_node_ns_def_get(VALUE self) {
+static VALUE
+rxml_node_ns_def_get(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
   if (xnode->nsDef == NULL)
     return(Qnil);
   else
-    return(ruby_xml_ns_wrap(xnode->nsDef));
+    return(rxml_ns_wrap(xnode->nsDef));
 }
 
 
@@ -1078,14 +1073,14 @@ ruby_xml_node_ns_def_get(VALUE self) {
  * 
  * Obtain this node's parent node, if any.
  */
-VALUE
-ruby_xml_node_parent_get(VALUE self) {
+static VALUE
+rxml_node_parent_get(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
 
   if (xnode->parent)
-    return(ruby_xml_node2_wrap(cXMLNode, xnode->parent));
+    return(rxml_node2_wrap(cXMLNode, xnode->parent));
   else
     return(Qnil);
 }
@@ -1097,8 +1092,8 @@ ruby_xml_node_parent_get(VALUE self) {
  * 
  * Obtain this node's path.
  */
-VALUE
-ruby_xml_node_path(VALUE self) {
+static VALUE
+rxml_node_path(VALUE self) {
   xmlNodePtr xnode;
   xmlChar *path;
 
@@ -1114,13 +1109,13 @@ ruby_xml_node_path(VALUE self) {
 
 /*
  * call-seq:
- *    node.pointer -> XML::Node_set
+ *    node.pointer -> XML::NodeSet
  * 
  * Evaluates an XPointer expression relative to this node.
  */
-VALUE
-ruby_xml_node_pointer(VALUE self, VALUE xptr_str) {
-  return(ruby_xml_xpointer_point2(self, xptr_str));
+static VALUE
+rxml_node_pointer(VALUE self, VALUE xptr_str) {
+  return(rxml_xpointer_point2(self, xptr_str));
 }
 
 
@@ -1130,8 +1125,8 @@ ruby_xml_node_pointer(VALUE self, VALUE xptr_str) {
  * 
  * Obtain the previous sibling, if any.
  */
-VALUE
-ruby_xml_node_prev_get(VALUE self) {
+static VALUE
+rxml_node_prev_get(VALUE self) {
   xmlNodePtr xnode;
   xmlNodePtr node;
   Data_Get_Struct(self, xmlNode, xnode);
@@ -1159,7 +1154,7 @@ ruby_xml_node_prev_get(VALUE self) {
   if (node == NULL)
     return(Qnil);
   else
-    return(ruby_xml_node2_wrap(cXMLNode, node));
+    return(rxml_node2_wrap(cXMLNode, node));
 }
 
 
@@ -1169,8 +1164,8 @@ ruby_xml_node_prev_get(VALUE self) {
  * 
  * Insert the specified node as this node's previous sibling.
  */
-VALUE
-ruby_xml_node_prev_set(VALUE self, VALUE rnode) {
+static VALUE
+rxml_node_prev_set(VALUE self, VALUE rnode) {
   xmlNodePtr cnode, pnode, ret;
 
   if (rb_obj_is_kind_of(rnode, cXMLNode) == Qfalse)
@@ -1181,9 +1176,9 @@ ruby_xml_node_prev_set(VALUE self, VALUE rnode) {
 
   ret = xmlAddPrevSibling(pnode, cnode);
   if (ret == NULL)
-    rb_raise(eXMLNodeFailedModify, "unable to add a sibling to the document");
+    rxml_raise(&xmlLastError);
 
-  return(ruby_xml_node2_wrap(cXMLNode, ret));
+  return(rxml_node2_wrap(cXMLNode, ret));
 }
 
 /*
@@ -1192,12 +1187,12 @@ ruby_xml_node_prev_set(VALUE self, VALUE rnode) {
  * 
  * Returns the XML::Attributes for this node. 
  */
-VALUE
-ruby_xml_node_attributes_get(VALUE self) {
+static VALUE
+rxml_node_attributes_get(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
-  return ruby_xml_attributes_new(xnode);
+  return rxml_attributes_new(xnode);
 }
 
 /*
@@ -1205,12 +1200,12 @@ ruby_xml_node_attributes_get(VALUE self) {
  *    node.property("name") -> "string"
  *    node["name"]          -> "string"
  * 
- * Obtain the named property.
+ * Obtain the named pyroperty.
  */
-VALUE
-ruby_xml_node_property_get(VALUE self, VALUE name) {
-  VALUE attributes = ruby_xml_node_attributes_get(self);
-  return ruby_xml_attributes_attribute_get(attributes, name);
+static VALUE
+rxml_node_attribute_get(VALUE self, VALUE name) {
+  VALUE attributes = rxml_node_attributes_get(self);
+  return rxml_attributes_attribute_get(attributes, name);
 }
 
 /*
@@ -1219,10 +1214,10 @@ ruby_xml_node_property_get(VALUE self, VALUE name) {
  * 
  * Set the named property.
  */
-VALUE
-ruby_xml_node_property_set(VALUE self, VALUE name, VALUE value) {
-  VALUE attributes = ruby_xml_node_attributes_get(self);
-  return ruby_xml_attributes_attribute_set(attributes, name, value);
+static VALUE
+rxml_node_property_set(VALUE self, VALUE name, VALUE value) {
+  VALUE attributes = rxml_node_attributes_get(self);
+  return rxml_attributes_attribute_set(attributes, name, value);
 }
 
 
@@ -1232,8 +1227,8 @@ ruby_xml_node_property_set(VALUE self, VALUE name, VALUE value) {
 *
 * Removes this node from it's parent.
 */
-VALUE
-ruby_xml_node_remove_ex(VALUE self) {
+static VALUE
+rxml_node_remove_ex(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
   xmlUnlinkNode(xnode);
@@ -1247,13 +1242,13 @@ ruby_xml_node_remove_ex(VALUE self) {
  * 
  * Search for a namespace by href.
  */
-VALUE
-ruby_xml_node_search_href(VALUE self, VALUE href) {
+static VALUE
+rxml_node_search_href(VALUE self, VALUE href) {
   xmlNodePtr xnode;
 
   Check_Type(href, T_STRING);
   Data_Get_Struct(self, xmlNode, xnode);
-  return(ruby_xml_ns_wrap(xmlSearchNsByHref(xnode->doc, xnode,
+  return(rxml_ns_wrap(xmlSearchNsByHref(xnode->doc, xnode,
 					    (xmlChar*)StringValuePtr(href))));
 }
 
@@ -1264,13 +1259,13 @@ ruby_xml_node_search_href(VALUE self, VALUE href) {
  * 
  * Search for a namespace by namespace.
  */
-VALUE
-ruby_xml_node_search_ns(VALUE self, VALUE ns) {
+static VALUE
+rxml_node_search_ns(VALUE self, VALUE ns) {
   xmlNodePtr xnode;
 
   Check_Type(ns, T_STRING);
   Data_Get_Struct(self, xmlNode, xnode);
-  return(ruby_xml_ns_wrap(xmlSearchNs(xnode->doc, xnode,
+  return(rxml_ns_wrap(xmlSearchNs(xnode->doc, xnode,
 			              (xmlChar*)StringValuePtr(ns))));
 }
 
@@ -1281,8 +1276,8 @@ ruby_xml_node_search_ns(VALUE self, VALUE ns) {
  * 
  * Add the specified node as a sibling of this node.
  */
-VALUE
-ruby_xml_node_sibling_set(VALUE self, VALUE rnode) {
+static VALUE
+rxml_node_sibling_set(VALUE self, VALUE rnode) {
   xmlNodePtr cnode, pnode, ret;
   VALUE obj;
 
@@ -1294,9 +1289,10 @@ ruby_xml_node_sibling_set(VALUE self, VALUE rnode) {
 
   ret = xmlAddSibling(pnode, cnode);
   if (ret == NULL)
-    rb_raise(eXMLNodeFailedModify, "unable to add a sibling to the document");
+    rxml_raise(&xmlLastError);
+
   if (ret->_private==NULL)
-    obj=ruby_xml_node2_wrap(cXMLNode,ret);
+    obj=rxml_node2_wrap(cXMLNode,ret);
   else
     obj=(VALUE)ret->_private;
 
@@ -1310,8 +1306,8 @@ ruby_xml_node_sibling_set(VALUE self, VALUE rnode) {
  * 
  * Determine whether this node preserves whitespace.
  */
-VALUE
-ruby_xml_node_space_preserve_get(VALUE self) {
+static VALUE
+rxml_node_space_preserve_get(VALUE self) {
   xmlNodePtr xnode;
 
   Data_Get_Struct(self, xmlNode, xnode);
@@ -1325,8 +1321,8 @@ ruby_xml_node_space_preserve_get(VALUE self) {
  * 
  * Control whether this node preserves whitespace.
  */
-VALUE
-ruby_xml_node_space_preserve_set(VALUE self, VALUE bool) {
+static VALUE
+rxml_node_space_preserve_set(VALUE self, VALUE bool) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
 
@@ -1345,8 +1341,8 @@ ruby_xml_node_space_preserve_set(VALUE self, VALUE bool) {
  * Coerce this node to a string representation of
  * it's XML.
  */
-VALUE
-ruby_xml_node_to_s(VALUE self) {
+static VALUE
+rxml_node_to_s(VALUE self) {
   xmlNodePtr xnode;
   xmlBufferPtr buf;
   VALUE result;
@@ -1367,8 +1363,8 @@ ruby_xml_node_to_s(VALUE self) {
  * 
  * Obtain this node's type identifier.
  */
-VALUE
-ruby_xml_node_type(VALUE self) {
+static VALUE
+rxml_node_type(VALUE self) {
   xmlNodePtr xnode;
   Data_Get_Struct(self, xmlNode, xnode);
   return(INT2NUM(xnode->type));
@@ -1387,8 +1383,8 @@ ruby_xml_node_type(VALUE self) {
  * to true. 
  *
  */
-VALUE
-ruby_xml_node_copy(VALUE self, VALUE deep) {
+static VALUE
+rxml_node_copy(VALUE self, VALUE deep) {
   xmlNodePtr xnode;
   xmlNodePtr xcopy;
   int recursive = (deep==Qnil || deep==Qfalse) ? 0 : 1;
@@ -1397,7 +1393,7 @@ ruby_xml_node_copy(VALUE self, VALUE deep) {
   xcopy = xmlCopyNode(xnode, recursive);
 
   if (xcopy)
-    return ruby_xml_node2_wrap(cXMLNode, xcopy);
+    return rxml_node2_wrap(cXMLNode, xcopy);
   else
     return Qnil;
 }
@@ -1410,8 +1406,8 @@ ruby_xml_node_copy(VALUE self, VALUE deep) {
  * the node's content.
  * 
  */
-VALUE
-ruby_xml_node_new_text(VALUE class, VALUE text)
+static VALUE
+rxml_node_new_text(VALUE class, VALUE text)
 {
   VALUE obj;
   xmlNodePtr xnode;
@@ -1426,19 +1422,19 @@ ruby_xml_node_new_text(VALUE class, VALUE text)
   if ( xnode == NULL )
     return Qnil;
   
-  obj=ruby_xml_node2_wrap(class,xnode);
+  obj=rxml_node2_wrap(class,xnode);
   rb_obj_call_init(obj,0,NULL);
   return obj;
 }
 
 void
-ruby_xml_node_registerNode(xmlNodePtr node)
+rxml_node_registerNode(xmlNodePtr node)
 {
   node->_private=NULL;
 }
 
 void
-ruby_xml_node_deregisterNode(xmlNodePtr xnode)
+rxml_node_deregisterNode(xmlNodePtr xnode)
 {
   VALUE node;
   
@@ -1455,13 +1451,10 @@ ruby_xml_node_deregisterNode(xmlNodePtr xnode)
 
 void
 ruby_init_xml_node(void) {
-  xmlRegisterNodeDefault(ruby_xml_node_registerNode);
-  xmlDeregisterNodeDefault(ruby_xml_node_deregisterNode);
+  xmlRegisterNodeDefault(rxml_node_registerNode);
+  xmlDeregisterNodeDefault(rxml_node_deregisterNode);
 
   cXMLNode = rb_define_class_under(mXML, "Node", rb_cObject);
-  eXMLNodeSetNamespace = rb_define_class_under(cXMLNode, "SetNamespace", eXMLError);
-  eXMLNodeFailedModify = rb_define_class_under(cXMLNode, "FailedModify", eXMLError);
-  eXMLNodeUnknownType = rb_define_class_under(cXMLNode, "UnknownType", eXMLError);
   
   rb_define_const(cXMLNode, "SPACE_DEFAULT", INT2NUM(0));
   rb_define_const(cXMLNode, "SPACE_PRESERVE", INT2NUM(1));
@@ -1505,78 +1498,69 @@ ruby_init_xml_node(void) {
   rb_define_const(cXMLNode, "DOCB_DOCUMENT_NODE", Qnil);
 #endif
   
-
-  rb_define_singleton_method(cXMLNode, "new2", ruby_xml_node2_new_native, 2);
-  rb_define_singleton_method(cXMLNode, "new", ruby_xml_node2_new_string_bc, -1);
-  rb_define_singleton_method(cXMLNode, "new_cdata", ruby_xml_node_new_cdata, -1);
-  rb_define_singleton_method(cXMLNode, "new_comment", ruby_xml_node_new_comment, -1); 
-  rb_define_singleton_method(cXMLNode, "new_text", ruby_xml_node_new_text, 1); 
+  rb_define_singleton_method(cXMLNode, "new", rxml_node_new_string_bc, -1);
+  rb_define_singleton_method(cXMLNode, "new_cdata", rxml_node_new_cdata, -1);
+  rb_define_singleton_method(cXMLNode, "new_comment", rxml_node_new_comment, -1); 
+  rb_define_singleton_method(cXMLNode, "new_text", rxml_node_new_text, 1); 
   
   /* Traversal */
   rb_include_module(cXMLNode, rb_mEnumerable);
-  rb_define_method(cXMLNode, "[]", ruby_xml_node_property_get, 1);
-  rb_define_method(cXMLNode, "each", ruby_xml_node_each, 0);
-  rb_define_method(cXMLNode, "first", ruby_xml_node_first_get, 0);
-  rb_define_method(cXMLNode, "last", ruby_xml_node_last_get, 0);
-  rb_define_method(cXMLNode, "next", ruby_xml_node_next_get, 0);
-  rb_define_method(cXMLNode, "parent", ruby_xml_node_parent_get, 0);
-  rb_define_method(cXMLNode, "prev", ruby_xml_node_prev_get, 0);
+  rb_define_method(cXMLNode, "[]", rxml_node_attribute_get, 1);
+  rb_define_method(cXMLNode, "each", rxml_node_each, 0);
+  rb_define_method(cXMLNode, "first", rxml_node_first_get, 0);
+  rb_define_method(cXMLNode, "last", rxml_node_last_get, 0);
+  rb_define_method(cXMLNode, "next", rxml_node_next_get, 0);
+  rb_define_method(cXMLNode, "parent", rxml_node_parent_get, 0);
+  rb_define_method(cXMLNode, "prev", rxml_node_prev_get, 0);
 
   /* Modification */
-  rb_define_method(cXMLNode, "<<", ruby_xml_node_content_add, 1);
-  rb_define_method(cXMLNode, "[]=", ruby_xml_node_property_set, 2);
-  rb_define_method(cXMLNode, "child_add", ruby_xml_node_child_add, 1);
-  rb_define_method(cXMLNode, "child=", ruby_xml_node_child_set, 1);
-  rb_define_method(cXMLNode, "sibling=", ruby_xml_node_sibling_set, 1);
-  rb_define_method(cXMLNode, "next=", ruby_xml_node_next_set, 1);
-  rb_define_method(cXMLNode, "prev=", ruby_xml_node_prev_set, 1);
+  rb_define_method(cXMLNode, "<<", rxml_node_content_add, 1);
+  rb_define_method(cXMLNode, "[]=", rxml_node_property_set, 2);
+  rb_define_method(cXMLNode, "child_add", rxml_node_child_add, 1);
+  rb_define_method(cXMLNode, "child=", rxml_node_child_set, 1);
+  rb_define_method(cXMLNode, "sibling=", rxml_node_sibling_set, 1);
+  rb_define_method(cXMLNode, "next=", rxml_node_next_set, 1);
+  rb_define_method(cXMLNode, "prev=", rxml_node_prev_set, 1);
 
  
   /* Rest of the node api */
-  rb_define_method(cXMLNode, "attributes", ruby_xml_node_attributes_get, 0);
-  rb_define_method(cXMLNode, "base", ruby_xml_node_base_get, 0);
-  rb_define_method(cXMLNode, "base=", ruby_xml_node_base_set, 1);
-  rb_define_method(cXMLNode, "blank?", ruby_xml_node_empty_q, 0);
-  rb_define_method(cXMLNode, "copy", ruby_xml_node_copy, 1);
-  rb_define_method(cXMLNode, "content", ruby_xml_node_content_get, 0);
-  rb_define_method(cXMLNode, "content=", ruby_xml_node_content_set, 1);
-  rb_define_method(cXMLNode, "content_stripped", ruby_xml_node_content_stripped_get, 0);
-  rb_define_method(cXMLNode, "doc", ruby_xml_node_doc, 0);
-  rb_define_method(cXMLNode, "dump", ruby_xml_node_dump, 0);
-  rb_define_method(cXMLNode, "debug_dump", ruby_xml_node_debug_dump, 0);
-  rb_define_method(cXMLNode, "empty?", ruby_xml_node_empty_q, 0);
-  rb_define_method(cXMLNode, "eql?", ruby_xml_node_eql_q, 1);
-  rb_define_method(cXMLNode, "lang", ruby_xml_node_lang_get, 0);
-  rb_define_method(cXMLNode, "lang=", ruby_xml_node_lang_set, 1);
-  rb_define_method(cXMLNode, "line_num", ruby_xml_node_line_num, 0);
-  rb_define_method(cXMLNode, "name", ruby_xml_node_name_get, 0);
-  rb_define_method(cXMLNode, "name=", ruby_xml_node_name_set, 1);
-  rb_define_method(cXMLNode, "namespace", ruby_xml_node_namespace_get, 0);
-  rb_define_method(cXMLNode, "namespace_node", ruby_xml_node_namespace_get_node, 0);
-  rb_define_method(cXMLNode, "namespace=", ruby_xml_node_namespace_set, -1);  
-  rb_define_method(cXMLNode, "node_type", ruby_xml_node_type, 0);
-  rb_define_method(cXMLNode, "ns", ruby_xml_node_namespace_get, 0);
-  rb_define_method(cXMLNode, "ns?", ruby_xml_node_ns_q, 0);
-  rb_define_method(cXMLNode, "ns_def?", ruby_xml_node_ns_def_q, 0);
-  rb_define_method(cXMLNode, "ns_def", ruby_xml_node_ns_def_get, 0);
-  rb_define_method(cXMLNode, "path", ruby_xml_node_path, 0);
-  rb_define_method(cXMLNode, "pointer", ruby_xml_node_pointer, 1);
-  rb_define_method(cXMLNode, "remove!", ruby_xml_node_remove_ex, 0);
-  rb_define_method(cXMLNode, "search_ns", ruby_xml_node_search_ns, 1);
-  rb_define_method(cXMLNode, "search_href", ruby_xml_node_search_href, 1);
-  rb_define_method(cXMLNode, "space_preserve", ruby_xml_node_space_preserve_get, 0);
-  rb_define_method(cXMLNode, "space_preserve=", ruby_xml_node_space_preserve_set, 1);
-  rb_define_method(cXMLNode, "to_s", ruby_xml_node_to_s, 0);
-  rb_define_method(cXMLNode, "xlink?", ruby_xml_node_xlink_q, 0);
-  rb_define_method(cXMLNode, "xlink_type", ruby_xml_node_xlink_type, 0);
-  rb_define_method(cXMLNode, "xlink_type_name", ruby_xml_node_xlink_type_name, 0);
-  
+  rb_define_method(cXMLNode, "attributes", rxml_node_attributes_get, 0);
+  rb_define_method(cXMLNode, "base", rxml_node_base_get, 0);
+  rb_define_method(cXMLNode, "base=", rxml_node_base_set, 1);
+  rb_define_method(cXMLNode, "blank?", rxml_node_empty_q, 0);
+  rb_define_method(cXMLNode, "copy", rxml_node_copy, 1);
+  rb_define_method(cXMLNode, "content", rxml_node_content_get, 0);
+  rb_define_method(cXMLNode, "content=", rxml_node_content_set, 1);
+  rb_define_method(cXMLNode, "content_stripped", rxml_node_content_stripped_get, 0);
+  rb_define_method(cXMLNode, "doc", rxml_node_doc, 0);
+  rb_define_method(cXMLNode, "dump", rxml_node_dump, 0);
+  rb_define_method(cXMLNode, "debug_dump", rxml_node_debug_dump, 0);
+  rb_define_method(cXMLNode, "empty?", rxml_node_empty_q, 0);
+  rb_define_method(cXMLNode, "eql?", rxml_node_eql_q, 1);
+  rb_define_method(cXMLNode, "lang", rxml_node_lang_get, 0);
+  rb_define_method(cXMLNode, "lang=", rxml_node_lang_set, 1);
+  rb_define_method(cXMLNode, "line_num", rxml_node_line_num, 0);
+  rb_define_method(cXMLNode, "name", rxml_node_name_get, 0);
+  rb_define_method(cXMLNode, "name=", rxml_node_name_set, 1);
+  rb_define_method(cXMLNode, "namespace", rxml_node_namespace_get, 0);
+  rb_define_method(cXMLNode, "namespace_node", rxml_node_namespace_get_node, 0);
+  rb_define_method(cXMLNode, "namespace=", rxml_node_namespace_set, -1);  
+  rb_define_method(cXMLNode, "node_type", rxml_node_type, 0);
+  rb_define_method(cXMLNode, "ns", rxml_node_namespace_get, 0);
+  rb_define_method(cXMLNode, "ns?", rxml_node_ns_q, 0);
+  rb_define_method(cXMLNode, "ns_def?", rxml_node_ns_def_q, 0);
+  rb_define_method(cXMLNode, "ns_def", rxml_node_ns_def_get, 0);
+  rb_define_method(cXMLNode, "path", rxml_node_path, 0);
+  rb_define_method(cXMLNode, "pointer", rxml_node_pointer, 1);
+  rb_define_method(cXMLNode, "remove!", rxml_node_remove_ex, 0);
+  rb_define_method(cXMLNode, "search_ns", rxml_node_search_ns, 1);
+  rb_define_method(cXMLNode, "search_href", rxml_node_search_href, 1);
+  rb_define_method(cXMLNode, "space_preserve", rxml_node_space_preserve_get, 0);
+  rb_define_method(cXMLNode, "space_preserve=", rxml_node_space_preserve_set, 1);
+  rb_define_method(cXMLNode, "to_s", rxml_node_to_s, 0);
+  rb_define_method(cXMLNode, "xlink?", rxml_node_xlink_q, 0);
+  rb_define_method(cXMLNode, "xlink_type", rxml_node_xlink_type, 0);
+  rb_define_method(cXMLNode, "xlink_type_name", rxml_node_xlink_type_name, 0);
+
   rb_define_alias(cXMLNode, "==", "eql?");
-  
-  /* :nodoc: */
-  {
-    VALUE singleton;
-    singleton = rb_singleton_class(cXMLNode);
-    rb_define_alias(singleton, "new_element", "new");
-  }
 }
