@@ -4,42 +4,55 @@
 #include "ruby_libxml.h"
 #include "ruby_xml_reader.h"
 
-#define CSTR2RVAL(x)  (x == NULL ? Qnil : rb_str_new2((const char *)x))
-#define RVAL2CSTR(x)  (StringValueCStr(x))
-
-static inline VALUE 
-__rb_str_new_and_free(xmlChar *x)
-{
-  if (x != NULL) {
-    VALUE v = rb_str_new2((const char *)x);
-    xmlFree(x);
-    return v;
-  }
-  return Qnil;
-}
-
-#define CSTR2RVAL2(x) (__rb_str_new_and_free(x))
-
 VALUE cXMLReader;
-static ID error_handler_block_ivar_id;
 
-static int
-ctxtRead(FILE *f, char * buf, size_t len) {
-    return(fread(buf, 1, len, f));
-}
+/*
+ * Document-class: LibXML::XML::Reader
+ *
+ * The XML::Reader class provides a simpler, alternative way of parsing an XML
+ * document in contrast to XML::Parser or XML::SaxParser.  A XML::Reader instance
+ * acts like a cursor going forward in a document stream, stopping at each node 
+ * it encounters.  To advance to the next node, simply cadd XML::Reader#read.
+ * 
+ * The XML::Reader API closely matches the DOM Core specification and supports
+ * namespaces, xml:base, entity handling and DTDs.  
+ *
+ * To summarize, XML::Reader provides a far simpler API to use versus XML::SaxParser
+ * and is more memory efficient than using XML::Parser to create a DOM tree.
+ *
+ * Example:
+ *
+ *  parser = XML::Reader.new("<foo><bar>1</bar><bar>2</bar><bar>3</bar></foo>")
+ *  parser.read
+ *  assert_equal('foo', parser.name)
+ *  assert_equal(nil, parser.value)
+ *  
+ *  3.times do |i|
+ *    parser.read
+ *    assert_equal(XML::Reader::TYPE_ELEMENT, parser.node_type)
+ *    assert_equal('bar', parser.name)
+ *    parser.read
+ *    assert_equal(XML::Reader::TYPE_TEXT, parser.node_type)
+ *    assert_equal((i + 1).to_s, parser.value)
+ *    parser.read
+ *    assert_equal(XML::Reader::TYPE_END_ELEMENT, parser.node_type)
+ *  end
+ *
+ * For a more in depth tutorial, albeit in C, see http://xmlsoft.org/xmlreader.html.*/
+
 
 static VALUE
-ruby_xml_reader_new(VALUE class, xmlTextReaderPtr reader)
+rxml_reader_new(VALUE class, xmlTextReaderPtr reader)
 {
   return Data_Wrap_Struct(class, NULL, xmlFreeTextReader, reader);
 }
 
 static xmlTextReaderPtr
-ruby_xml_text_reader_get(VALUE obj)
+rxml_text_reader_get(VALUE obj)
 {
-  xmlTextReaderPtr ptr;
-  Data_Get_Struct(obj, xmlTextReader, ptr);
-  return ptr;
+  xmlTextReaderPtr xreader;
+  Data_Get_Struct(obj, xmlTextReader, xreader);
+  return xreader;
 }
 
 /*
@@ -50,22 +63,26 @@ ruby_xml_text_reader_get(VALUE obj)
  * options are a combination of xmlParserOption. 
  */
 static VALUE
-ruby_xml_reader_new_file(int argc, VALUE *argv, VALUE self)
+rxml_reader_new_file(int argc, VALUE *argv, VALUE self)
 {
-  xmlTextReaderPtr reader;
-  VALUE path, encoding, options;
+  xmlTextReaderPtr xreader;
+  VALUE rpath, rencoding, roptions;
+  char *xpath;
+  char *xencoding;
+  int options;
 
-  rb_scan_args(argc, argv, "12", &path, &encoding, &options);
+  rb_scan_args(argc, argv, "12", &rpath, &rencoding, &roptions);
 
-  reader = xmlReaderForFile(RVAL2CSTR(path), 
-                            NIL_P(encoding) ? NULL : RVAL2CSTR(encoding), 
-                            NIL_P(options) ? 0 : FIX2INT(options));
-  if (reader == NULL)
-    rb_raise(rb_eRuntimeError, 
-             "cannot create text reader for given XML file at path '%s'", 
-             RVAL2CSTR(path));
+  xpath = NIL_P(rpath) ? NULL : StringValueCStr(rpath);
+  xencoding = NIL_P(rencoding) ? NULL : StringValueCStr(rencoding);
+  options = NIL_P(roptions) ? 0 : FIX2INT(roptions);
 
-  return ruby_xml_reader_new(self, reader); 
+  xreader = xmlReaderForFile(xpath, xencoding, options);
+
+  if (xreader == NULL)
+    rxml_raise(&xmlLastError);
+
+  return rxml_reader_new(self, xreader); 
 }
 
 /*
@@ -76,34 +93,28 @@ ruby_xml_reader_new_file(int argc, VALUE *argv, VALUE self)
  * a combination of xmlParserOption. 
  */
 static VALUE
-ruby_xml_reader_new_io(int argc, VALUE *argv, VALUE self)
+rxml_reader_new_io(int argc, VALUE *argv, VALUE self)
 {
-  xmlTextReaderPtr reader;
-  VALUE io, url, encoding, options;
-  OpenFile *fptr;
-  FILE *f;
+  xmlTextReaderPtr xreader;
+  VALUE rio, rurl, rencoding, roptions;
+  char *xurl;
+  char *xencoding;
+  int options;
 
-  #ifdef _WIN32
-    rb_raise(rb_eRuntimeError, "Reading an io stream is not supported on Windows");
-  #endif             
+  rb_scan_args(argc, argv, "13", &rio, &rurl, &rencoding, &roptions);
 
-  rb_scan_args(argc, argv, "13", &io, &url, &encoding, &options);
+  xurl = NIL_P(rurl) ? NULL : StringValueCStr(rurl);
+  xencoding = NIL_P(rencoding) ? NULL : StringValueCStr(rencoding);
+  options = NIL_P(roptions) ? 0 : FIX2INT(roptions);
 
-  if (!rb_obj_is_kind_of(io, rb_cIO))
-    rb_raise(rb_eTypeError, "need an IO object");
+  xreader = xmlReaderForIO((xmlInputReadCallback) rxml_read_callback, NULL,
+                           (void *)rio,
+                           xurl, xencoding, options);
 
-  GetOpenFile(io, fptr);
-  rb_io_check_readable(fptr);
-  f = GetWriteFile(fptr);
+  if (xreader == NULL)
+    rxml_raise(&xmlLastError);
 
-  reader = xmlReaderForIO((xmlInputReadCallback) ctxtRead, NULL, f,
-                          NIL_P(url) ? NULL : RVAL2CSTR(url),
-                          NIL_P(encoding) ? NULL : RVAL2CSTR(encoding), 
-                          NIL_P(options) ? 0 : FIX2INT(options));
-  if (reader == NULL)
-    rb_raise(rb_eRuntimeError, "cannot create text reader for given stream");
-
-  return ruby_xml_reader_new(self, reader); 
+  return rxml_reader_new(self, xreader); 
 }
 
 /*
@@ -114,18 +125,19 @@ ruby_xml_reader_new_io(int argc, VALUE *argv, VALUE self)
  * Create an XML text reader for a preparsed document.
  */
 VALUE
-ruby_xml_reader_new_walker(VALUE self, VALUE doc)
+rxml_reader_new_walker(VALUE self, VALUE doc)
 {
-  ruby_xml_document_t *rxd;
-  xmlTextReaderPtr reader;
+  xmlDocPtr xdoc;
+  xmlTextReaderPtr xreader;
   
-  Data_Get_Struct(doc, ruby_xml_document_t, rxd);
+  Data_Get_Struct(doc, xmlDoc, xdoc);
  
-  reader = xmlReaderWalker(rxd->doc);  
-  if (reader == NULL)
-    rb_raise(rb_eRuntimeError, "cannot create text reader for given document");
+  xreader = xmlReaderWalker(xdoc);  
 
-  return ruby_xml_reader_new(self, reader); 
+  if (xreader == NULL)
+    rxml_raise(&xmlLastError);
+
+  return rxml_reader_new(self, xreader); 
 }
 
 /*
@@ -137,24 +149,29 @@ ruby_xml_reader_new_walker(VALUE self, VALUE doc)
  * options are a combination of xmlParserOption.
  */
 static VALUE
-ruby_xml_reader_new_data(int argc, VALUE *argv, VALUE self)
+rxml_reader_new_data(int argc, VALUE *argv, VALUE self)
 {
-  xmlTextReaderPtr reader;
-  VALUE data, url, encoding, options;
-  char *c_data;
+  xmlTextReaderPtr xreader;
+  VALUE rdata, rurl, rencoding, roptions;
+  char *xdata;
+  char *xurl;
+  char *xencoding;
+  int options;
 
-  rb_scan_args(argc, argv, "13", &data, &url, &encoding, &options);
+  rb_scan_args(argc, argv, "13", &rdata, &rurl, &rencoding, &roptions);
 
-  c_data = RVAL2CSTR(data);
-  reader = xmlReaderForMemory(c_data,
-                              strlen(c_data), 
-                              NIL_P(url) ? NULL : RVAL2CSTR(url),
-                              NIL_P(encoding) ? NULL : RVAL2CSTR(encoding), 
-                              NIL_P(options) ? 0 : FIX2INT(options));
-  if (reader == NULL)
-    rb_raise(rb_eRuntimeError, "cannot create text reader for given data");
+  xdata = NIL_P(rdata) ? NULL : StringValueCStr(rdata);
+  xurl = NIL_P(rurl) ? NULL : StringValueCStr(rurl);
+  xencoding = NIL_P(rencoding) ? NULL : StringValueCStr(rencoding);
+  options = NIL_P(roptions) ? 0 : FIX2INT(roptions);
 
-  return ruby_xml_reader_new(self, reader); 
+  xreader = xmlReaderForMemory(xdata, strlen(xdata), 
+                              xurl, xencoding, options);
+
+  if (xreader == NULL)
+    rxml_raise(&xmlLastError);
+
+  return rxml_reader_new(self, xreader); 
 }
 
 /*
@@ -165,9 +182,9 @@ ruby_xml_reader_new_data(int argc, VALUE *argv, VALUE self)
  * changes the state to Closed and close any underlying input.
  */
 static VALUE
-ruby_xml_reader_close(VALUE self)
+rxml_reader_close(VALUE self)
 {
-  return INT2FIX(xmlTextReaderClose(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderClose(rxml_text_reader_get(self)));
 }
 
 /*
@@ -179,18 +196,18 @@ ruby_xml_reader_close(VALUE self)
  * relative to the containing element.
  */
 static VALUE
-ruby_xml_reader_move_to_attr(VALUE self, VALUE val)
+rxml_reader_move_to_attr(VALUE self, VALUE val)
 {
-  xmlTextReaderPtr reader;
+  xmlTextReaderPtr xreader;
   int ret;
 
-  reader = ruby_xml_text_reader_get(self);
+  xreader = rxml_text_reader_get(self);
 
   if (TYPE(val) == T_FIXNUM) {
-    ret = xmlTextReaderMoveToAttributeNo(reader, FIX2INT(val));
+    ret = xmlTextReaderMoveToAttributeNo(xreader, FIX2INT(val));
   }
   else {
-    ret = xmlTextReaderMoveToAttribute(reader, (const xmlChar *)RVAL2CSTR(val));
+    ret = xmlTextReaderMoveToAttribute(xreader, (const xmlChar *)StringValueCStr(val));
   }
 
   return INT2FIX(ret);
@@ -204,9 +221,9 @@ ruby_xml_reader_move_to_attr(VALUE self, VALUE val)
  * with the current node.
  */
 static VALUE
-ruby_xml_reader_move_to_first_attr(VALUE self)
+rxml_reader_move_to_first_attr(VALUE self)
 {
-  return INT2FIX(xmlTextReaderMoveToFirstAttribute(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderMoveToFirstAttribute(rxml_text_reader_get(self)));
 }
 
 /*
@@ -217,9 +234,9 @@ ruby_xml_reader_move_to_first_attr(VALUE self)
  * with the current node.
  */
 static VALUE
-ruby_xml_reader_move_to_next_attr(VALUE self)
+rxml_reader_move_to_next_attr(VALUE self)
 {
-  return INT2FIX(xmlTextReaderMoveToNextAttribute(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderMoveToNextAttribute(rxml_text_reader_get(self)));
 }
 
 /*
@@ -230,9 +247,9 @@ ruby_xml_reader_move_to_next_attr(VALUE self)
  * current attribute node.
  */
 static VALUE
-ruby_xml_reader_move_to_element(VALUE self)
+rxml_reader_move_to_element(VALUE self)
 {
-  return INT2FIX(xmlTextReaderMoveToElement(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderMoveToElement(rxml_text_reader_get(self)));
 }
 
 /*
@@ -243,9 +260,9 @@ ruby_xml_reader_move_to_element(VALUE self)
  * the subtree if any.
  */
 static VALUE
-ruby_xml_reader_next(VALUE self)
+rxml_reader_next(VALUE self)
 {
-  return INT2FIX(xmlTextReaderNext(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderNext(rxml_text_reader_get(self)));
 }
 
 /*
@@ -257,9 +274,9 @@ ruby_xml_reader_next(VALUE self)
  * document.
  */
 static VALUE
-ruby_xml_reader_next_sibling(VALUE self)
+rxml_reader_next_sibling(VALUE self)
 {
-  return INT2FIX(xmlTextReaderNextSibling(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderNextSibling(rxml_text_reader_get(self)));
 }
 
 /*
@@ -270,9 +287,9 @@ ruby_xml_reader_next_sibling(VALUE self)
  * http://dotgnu.org/pnetlib-doc/System/Xml/XmlNodeType.html
  */
 static VALUE
-ruby_xml_reader_node_type(VALUE self)
+rxml_reader_node_type(VALUE self)
 {
-  return INT2FIX(xmlTextReaderNodeType(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderNodeType(rxml_text_reader_get(self)));
 }
 
 /*
@@ -288,9 +305,9 @@ ruby_xml_reader_node_type(VALUE self)
  * Return 1 or -1 in case of error.
  */
 static VALUE
-ruby_xml_reader_normalization(VALUE self)
+rxml_reader_normalization(VALUE self)
 {
-  return INT2FIX(xmlTextReaderNormalization(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderNormalization(rxml_text_reader_get(self)));
 }
 
 /*
@@ -304,9 +321,9 @@ ruby_xml_reader_normalization(VALUE self)
  * read, or -1 in case of error.
  */
 static VALUE
-ruby_xml_reader_read(VALUE self)
+rxml_reader_read(VALUE self)
 {
-  return INT2FIX(xmlTextReaderRead(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderRead(rxml_text_reader_get(self)));
 }
 
 /*
@@ -320,9 +337,9 @@ ruby_xml_reader_read(VALUE self)
  * error.
  */
 static VALUE
-ruby_xml_reader_read_attr_value(VALUE self)
+rxml_reader_read_attr_value(VALUE self)
 {
-  return INT2FIX(xmlTextReaderReadAttributeValue(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderReadAttributeValue(rxml_text_reader_get(self)));
 }
 
 /*
@@ -335,9 +352,10 @@ ruby_xml_reader_read_attr_value(VALUE self)
  * neither an element nor attribute, or has no child nodes. 
  */
 static VALUE
-ruby_xml_reader_read_inner_xml(VALUE self)
+rxml_reader_read_inner_xml(VALUE self)
 {
-  return CSTR2RVAL2(xmlTextReaderReadInnerXml(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderReadInnerXml(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -350,9 +368,10 @@ ruby_xml_reader_read_inner_xml(VALUE self)
  * neither an element nor attribute, or has no child nodes. 
  */
 static VALUE
-ruby_xml_reader_read_outer_xml(VALUE self)
+rxml_reader_read_outer_xml(VALUE self)
 {
-  return CSTR2RVAL2(xmlTextReaderReadOuterXml(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderReadOuterXml(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -362,9 +381,9 @@ ruby_xml_reader_read_outer_xml(VALUE self)
  * Get the read state of the reader.
  */
 static VALUE
-ruby_xml_reader_read_state(VALUE self)
+rxml_reader_read_state(VALUE self)
 {
-  return INT2FIX(xmlTextReaderReadState(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderReadState(rxml_text_reader_get(self)));
 }
 
 /*
@@ -377,74 +396,10 @@ ruby_xml_reader_read_state(VALUE self)
  * if the reader is positioned on any other type of node.
  */
 static VALUE
-ruby_xml_reader_read_string(VALUE self)
+rxml_reader_read_string(VALUE self)
 {
-  return CSTR2RVAL2(xmlTextReaderReadString(ruby_xml_text_reader_get(self)));
-}
-
-static void
-__xml_reader_error_cb(void *arg, 
-                      const char *msg, xmlParserSeverities severity, 
-                      xmlTextReaderLocatorPtr locator)
-{
-  VALUE reader;
-  VALUE block;
-
-  reader = (VALUE)arg;
-  block = rb_ivar_get(reader, error_handler_block_ivar_id); 
-  if (NIL_P(block))
-    rb_bug("no ivar block");
-  
-  rb_funcall(block, 
-             rb_intern("call"), 
-             5, 
-             reader, 
-             CSTR2RVAL(msg), 
-             INT2FIX(severity), 
-             CSTR2RVAL2(xmlTextReaderLocatorBaseURI(locator)), 
-             INT2FIX(xmlTextReaderLocatorLineNumber(locator))); 
-}
-
-/*
- * call-seq:
- *    reader.set_error_handler { ... }
- *
- * Register a callback block that will be called on error and warnings. The
- * block will be called with 5 parameters: the reader, the error message, the 
- * severity, the base URI and the line number.
- */
-static VALUE
-ruby_xml_reader_set_error_handler(VALUE self)
-{
-  VALUE block;
-  xmlTextReaderPtr reader;
-
-  if (rb_block_given_p() == Qfalse)
-    rb_raise(rb_eRuntimeError, "No block given");
-
-  block = rb_block_proc();
- 
-  /* Embed the block within the parser object to avoid it to be collected.
-   * Previous handler if exits is overwritten. 
-   */
-  rb_ivar_set(self, error_handler_block_ivar_id, block); 
-  reader = ruby_xml_text_reader_get(self);
-  xmlTextReaderSetErrorHandler(reader, __xml_reader_error_cb, (void *)self); 
-  
-  return self;
-}
-
-/*
- * call-seq:
- *    reader.reset_error_handler
- *
- * Restore the default error and warning handlers.
- */
-static VALUE
-ruby_xml_reader_reset_error_handler(VALUE self)
-{
-  xmlTextReaderSetErrorHandler(ruby_xml_text_reader_get(self), NULL, NULL);
-  return self;
+  const xmlChar *result = xmlTextReaderReadString(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -459,9 +414,10 @@ ruby_xml_reader_reset_error_handler(VALUE self)
  * case of error.
  */ 
 static VALUE
-ruby_xml_reader_relax_ng_validate(VALUE self, VALUE rng)
+rxml_reader_relax_ng_validate(VALUE self, VALUE rng)
 {
-  return INT2FIX(xmlTextReaderRelaxNGValidate(ruby_xml_text_reader_get(self), NIL_P(rng) ? NULL : RVAL2CSTR(rng)));
+  char *xrng = NIL_P(rng) ? NULL : StringValueCStr(rng);
+  return INT2FIX(xmlTextReaderRelaxNGValidate(rxml_text_reader_get(self), xrng));
 }
 
 #if LIBXML_VERSION >= 20620
@@ -477,9 +433,11 @@ ruby_xml_reader_relax_ng_validate(VALUE self, VALUE rng)
  * case of error.
  */
 static VALUE
-ruby_xml_reader_schema_validate(VALUE self, VALUE xsd)
+rxml_reader_schema_validate(VALUE self, VALUE xsd)
 {
-  return INT2FIX(xmlTextReaderSchemaValidate(ruby_xml_text_reader_get(self), NIL_P(xsd) ? NULL : RVAL2CSTR(xsd)));
+    char *xxsd = NIL_P(xsd) ? NULL : StringValueCStr(xsd);
+    int status = xmlTextReaderSchemaValidate(rxml_text_reader_get(self), xxsd);
+    return INT2FIX(status);
 }
 #endif
 
@@ -490,9 +448,10 @@ ruby_xml_reader_schema_validate(VALUE self, VALUE xsd)
  * Return the qualified name of the node.
  */
 static VALUE
-ruby_xml_reader_name(VALUE self)
+rxml_reader_name(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstName(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstName(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /* 
@@ -502,9 +461,10 @@ ruby_xml_reader_name(VALUE self)
  * Return the local name of the node.
  */
 static VALUE
-ruby_xml_reader_local_name(VALUE self)
+rxml_reader_local_name(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstLocalName(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstLocalName(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -514,9 +474,9 @@ ruby_xml_reader_local_name(VALUE self)
  * Provide the number of attributes of the current node.
  */
 static VALUE
-ruby_xml_reader_attr_count(VALUE self)
+rxml_reader_attr_count(VALUE self)
 {
-  return INT2FIX(xmlTextReaderAttributeCount(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderAttributeCount(rxml_text_reader_get(self)));
 }
 
 /*
@@ -526,9 +486,10 @@ ruby_xml_reader_attr_count(VALUE self)
  * Determine the encoding of the document being read.
  */
 static VALUE
-ruby_xml_reader_encoding(VALUE self)
+rxml_reader_encoding(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstEncoding(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstEncoding(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -538,9 +499,10 @@ ruby_xml_reader_encoding(VALUE self)
  * Determine the base URI of the node.
  */
 static VALUE
-ruby_xml_reader_base_uri(VALUE self)
+rxml_reader_base_uri(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstBaseUri(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstBaseUri(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -550,9 +512,10 @@ ruby_xml_reader_base_uri(VALUE self)
  * Determine the namespace URI of the node.
  */
 static VALUE
-ruby_xml_reader_namespace_uri(VALUE self)
+rxml_reader_namespace_uri(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstNamespaceUri(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstNamespaceUri(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -562,9 +525,10 @@ ruby_xml_reader_namespace_uri(VALUE self)
  * Provide the text value of the node if present.
  */
 static VALUE
-ruby_xml_reader_value(VALUE self)
+rxml_reader_value(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstValue(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstValue(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /* 
@@ -574,9 +538,10 @@ ruby_xml_reader_value(VALUE self)
  * Get a shorthand reference to the namespace associated with the node.
  */
 static VALUE
-ruby_xml_reader_prefix(VALUE self)
+rxml_reader_prefix(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstPrefix(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstPrefix(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -586,9 +551,9 @@ ruby_xml_reader_prefix(VALUE self)
  * Get the depth of the node in the tree.
  */
 static VALUE
-ruby_xml_reader_depth(VALUE self)
+rxml_reader_depth(VALUE self)
 {
-  return INT2FIX(xmlTextReaderDepth(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderDepth(rxml_text_reader_get(self)));
 }
 
 /*
@@ -599,9 +564,9 @@ ruby_xml_reader_depth(VALUE self)
  * as an integer value (and -1 in case of error).
  */
 static VALUE
-ruby_xml_reader_quote_char(VALUE self)
+rxml_reader_quote_char(VALUE self)
 {
-  return INT2FIX(xmlTextReaderQuoteChar(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderQuoteChar(rxml_text_reader_get(self)));
 }
 
 /*
@@ -615,9 +580,9 @@ ruby_xml_reader_quote_char(VALUE self)
  * standalone status or in case of error.
  */
 static VALUE
-ruby_xml_reader_standalone(VALUE self)
+rxml_reader_standalone(VALUE self)
 {
-  return INT2FIX(xmlTextReaderStandalone(ruby_xml_text_reader_get(self)));
+  return INT2FIX(xmlTextReaderStandalone(rxml_text_reader_get(self)));
 }
 
 /*
@@ -627,9 +592,10 @@ ruby_xml_reader_standalone(VALUE self)
  * Get the xml:lang scope within which the node resides.
  */
 static VALUE
-ruby_xml_reader_xml_lang(VALUE self)
+rxml_reader_xml_lang(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstXmlLang(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstXmlLang(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -639,9 +605,10 @@ ruby_xml_reader_xml_lang(VALUE self)
  * Determine the XML version of the document being read.
  */
 static VALUE
-ruby_xml_reader_xml_version(VALUE self)
+rxml_reader_xml_version(VALUE self)
 {
-  return CSTR2RVAL(xmlTextReaderConstXmlVersion(ruby_xml_text_reader_get(self)));
+  const xmlChar *result = xmlTextReaderConstXmlVersion(rxml_text_reader_get(self));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -651,9 +618,9 @@ ruby_xml_reader_xml_version(VALUE self)
  * Get whether the node has attributes.
  */
 static VALUE
-ruby_xml_reader_has_attributes(VALUE self)
+rxml_reader_has_attributes(VALUE self)
 {
-  return xmlTextReaderHasAttributes(ruby_xml_text_reader_get(self)) ? Qtrue : Qfalse;
+  return xmlTextReaderHasAttributes(rxml_text_reader_get(self)) ? Qtrue : Qfalse;
 }
 
 /*
@@ -663,9 +630,9 @@ ruby_xml_reader_has_attributes(VALUE self)
  * Get whether the node can have a text value.
  */
 static VALUE
-ruby_xml_reader_has_value(VALUE self)
+rxml_reader_has_value(VALUE self)
 {
-  return xmlTextReaderHasValue(ruby_xml_text_reader_get(self)) ? Qtrue : Qfalse;
+  return xmlTextReaderHasValue(rxml_text_reader_get(self)) ? Qtrue : Qfalse;
 }
 
 /*
@@ -677,20 +644,20 @@ ruby_xml_reader_has_value(VALUE self)
  * containing element, as a string.
  */
 static VALUE
-ruby_xml_reader_attribute(VALUE self, VALUE key)
+rxml_reader_attribute(VALUE self, VALUE key)
 {
   xmlTextReaderPtr reader;
   xmlChar *attr;
 
-  reader = ruby_xml_text_reader_get(self);
+  reader = rxml_text_reader_get(self);
 
   if (TYPE(key) == T_FIXNUM) {
     attr = xmlTextReaderGetAttributeNo(reader, FIX2INT(key));
   }
   else {
-    attr = xmlTextReaderGetAttribute(reader, (const xmlChar *)RVAL2CSTR(key));
+    attr = xmlTextReaderGetAttribute(reader, (const xmlChar *)StringValueCStr(key));
   }
-  return CSTR2RVAL2(attr);
+  return (attr == NULL ? Qnil : rb_str_new2(attr));
 }
 
 /*
@@ -701,9 +668,10 @@ ruby_xml_reader_attribute(VALUE self, VALUE key)
  * To return the default namespace, specify nil as +prefix+.
  */
 static VALUE
-ruby_xml_reader_lookup_namespace(VALUE self, VALUE prefix)
+rxml_reader_lookup_namespace(VALUE self, VALUE prefix)
 {
-  return CSTR2RVAL2(xmlTextReaderLookupNamespace(ruby_xml_text_reader_get(self), (const xmlChar *)RVAL2CSTR(prefix)));
+  const xmlChar *result = xmlTextReaderLookupNamespace(rxml_text_reader_get(self), (const xmlChar *)StringValueCStr(prefix));
+  return (result == NULL ? Qnil : rb_str_new2(result));
 }
 
 /*
@@ -716,11 +684,11 @@ ruby_xml_reader_lookup_namespace(VALUE self, VALUE prefix)
  * Return an XML::Node object, or nil in case of error.
  */
 static VALUE
-ruby_xml_reader_expand(VALUE self)
+rxml_reader_expand(VALUE self)
 {
   xmlNodePtr node;
   xmlDocPtr doc;
-  xmlTextReaderPtr reader = ruby_xml_text_reader_get(self);
+  xmlTextReaderPtr reader = rxml_text_reader_get(self);
   node = xmlTextReaderExpand(reader);
   
   if (!node)
@@ -734,9 +702,9 @@ ruby_xml_reader_expand(VALUE self)
 
   xmlTextReaderPreserve(reader);
   doc = xmlTextReaderCurrentDoc(reader);
-  ruby_xml_document_wrap(doc);
+  rxml_document_wrap(doc);
 
-  return ruby_xml_node2_wrap(cXMLNode, node);
+  return rxml_node2_wrap(cXMLNode, node);
 }
 
 #if LIBXML_VERSION >= 20618
@@ -748,9 +716,9 @@ ruby_xml_reader_expand(VALUE self)
  * relative to the start of the current entity. 
  */
 static VALUE
-ruby_xml_reader_byte_consumed(VALUE self)
+rxml_reader_byte_consumed(VALUE self)
 {
-  return INT2NUM(xmlTextReaderByteConsumed(ruby_xml_text_reader_get(self)));
+  return INT2NUM(xmlTextReaderByteConsumed(rxml_text_reader_get(self)));
 }
 #endif
 
@@ -762,9 +730,9 @@ ruby_xml_reader_byte_consumed(VALUE self)
  * Provide the column number of the current parsing point.
  */
 static VALUE
-ruby_xml_reader_column_number(VALUE self)
+rxml_reader_column_number(VALUE self)
 {
-  return INT2NUM(xmlTextReaderGetParserColumnNumber(ruby_xml_text_reader_get(self)));
+  return INT2NUM(xmlTextReaderGetParserColumnNumber(rxml_text_reader_get(self)));
 }
 
 /*
@@ -774,9 +742,9 @@ ruby_xml_reader_column_number(VALUE self)
  * Provide the line number of the current parsing point.
  */
 static VALUE
-ruby_xml_reader_line_number(VALUE self)
+rxml_reader_line_number(VALUE self)
 {
-  return INT2NUM(xmlTextReaderGetParserLineNumber(ruby_xml_text_reader_get(self)));
+  return INT2NUM(xmlTextReaderGetParserLineNumber(rxml_text_reader_get(self)));
 }
 #endif
 
@@ -788,9 +756,9 @@ ruby_xml_reader_line_number(VALUE self)
  * defined in the DTD or schema.
  */
 static VALUE
-ruby_xml_reader_default(VALUE self)
+rxml_reader_default(VALUE self)
 {
-  return xmlTextReaderIsDefault(ruby_xml_text_reader_get(self)) ? Qtrue : Qfalse;
+  return xmlTextReaderIsDefault(rxml_text_reader_get(self)) ? Qtrue : Qfalse;
 }
 
 /*
@@ -801,9 +769,9 @@ ruby_xml_reader_default(VALUE self)
  * regular attribute.
  */
 static VALUE
-ruby_xml_reader_namespace_declaration(VALUE self)
+rxml_reader_namespace_declaration(VALUE self)
 {
-  return xmlTextReaderIsNamespaceDecl(ruby_xml_text_reader_get(self)) ? Qtrue : Qfalse;
+  return xmlTextReaderIsNamespaceDecl(rxml_text_reader_get(self)) ? Qtrue : Qfalse;
 }
 
 /*
@@ -813,9 +781,9 @@ ruby_xml_reader_namespace_declaration(VALUE self)
  * Check if the current node is empty.
  */
 static VALUE
-ruby_xml_reader_empty_element(VALUE self)
+rxml_reader_empty_element(VALUE self)
 {
-  return xmlTextReaderIsEmptyElement(ruby_xml_text_reader_get(self)) ? Qtrue : Qfalse;
+  return xmlTextReaderIsEmptyElement(rxml_text_reader_get(self)) ? Qtrue : Qfalse;
 }
 
 /*
@@ -825,9 +793,9 @@ ruby_xml_reader_empty_element(VALUE self)
  * Retrieve the validity status from the parser context.
  */
 static VALUE
-ruby_xml_reader_valid(VALUE self)
+rxml_reader_valid(VALUE self)
 {
-  return xmlTextReaderIsValid(ruby_xml_text_reader_get(self)) ? Qtrue : Qfalse;
+  return xmlTextReaderIsValid(rxml_text_reader_get(self)) ? Qtrue : Qfalse;
 }
 
 /* Rdoc needs to know. */ 
@@ -840,72 +808,68 @@ void
 ruby_init_xml_reader(void)
 {
   cXMLReader = rb_define_class_under(mXML, "Reader", rb_cObject);
-  error_handler_block_ivar_id = rb_intern("@__error_handler_callback__"); 
  
-  rb_define_singleton_method(cXMLReader, "file", ruby_xml_reader_new_file, -1);
-  rb_define_singleton_method(cXMLReader, "io", ruby_xml_reader_new_io, -1);
-  rb_define_singleton_method(cXMLReader, "walker", ruby_xml_reader_new_walker, 1);
+  rb_define_singleton_method(cXMLReader, "file", rxml_reader_new_file, -1);
+  rb_define_singleton_method(cXMLReader, "io", rxml_reader_new_io, -1);
+  rb_define_singleton_method(cXMLReader, "walker", rxml_reader_new_walker, 1);
   rb_define_alias(CLASS_OF(cXMLReader), "document", "walker");
-  rb_define_singleton_method(cXMLReader, "new", ruby_xml_reader_new_data, -1);
+  rb_define_singleton_method(cXMLReader, "new", rxml_reader_new_data, -1);
   rb_define_alias(CLASS_OF(cXMLReader), "string", "new");
 
-  rb_define_method(cXMLReader, "close", ruby_xml_reader_close, 0);
+  rb_define_method(cXMLReader, "close", rxml_reader_close, 0);
 
-  rb_define_method(cXMLReader, "move_to_attribute", ruby_xml_reader_move_to_attr, 1);
-  rb_define_method(cXMLReader, "move_to_first_attribute", ruby_xml_reader_move_to_first_attr, 0);
-  rb_define_method(cXMLReader, "move_to_next_attribute", ruby_xml_reader_move_to_next_attr, 0);
-  rb_define_method(cXMLReader, "move_to_element", ruby_xml_reader_move_to_element, 0);
-  rb_define_method(cXMLReader, "next", ruby_xml_reader_next, 0);
-  rb_define_method(cXMLReader, "next_sibling", ruby_xml_reader_next_sibling, 0);
-  rb_define_method(cXMLReader, "read", ruby_xml_reader_read, 0);
-  rb_define_method(cXMLReader, "read_attribute_value", ruby_xml_reader_read_attr_value, 0);
-  rb_define_method(cXMLReader, "read_inner_xml", ruby_xml_reader_read_inner_xml, 0);
-  rb_define_method(cXMLReader, "read_outer_xml", ruby_xml_reader_read_outer_xml, 0);
-  rb_define_method(cXMLReader, "read_state", ruby_xml_reader_read_state, 0);
-  rb_define_method(cXMLReader, "read_string", ruby_xml_reader_read_string, 0);
+  rb_define_method(cXMLReader, "move_to_attribute", rxml_reader_move_to_attr, 1);
+  rb_define_method(cXMLReader, "move_to_first_attribute", rxml_reader_move_to_first_attr, 0);
+  rb_define_method(cXMLReader, "move_to_next_attribute", rxml_reader_move_to_next_attr, 0);
+  rb_define_method(cXMLReader, "move_to_element", rxml_reader_move_to_element, 0);
+  rb_define_method(cXMLReader, "next", rxml_reader_next, 0);
+  rb_define_method(cXMLReader, "next_sibling", rxml_reader_next_sibling, 0);
+  rb_define_method(cXMLReader, "read", rxml_reader_read, 0);
+  rb_define_method(cXMLReader, "read_attribute_value", rxml_reader_read_attr_value, 0);
+  rb_define_method(cXMLReader, "read_inner_xml", rxml_reader_read_inner_xml, 0);
+  rb_define_method(cXMLReader, "read_outer_xml", rxml_reader_read_outer_xml, 0);
+  rb_define_method(cXMLReader, "read_state", rxml_reader_read_state, 0);
+  rb_define_method(cXMLReader, "read_string", rxml_reader_read_string, 0);
 
-  rb_define_method(cXMLReader, "set_error_handler", ruby_xml_reader_set_error_handler, 0);
-  rb_define_method(cXMLReader, "reset_error_handler", ruby_xml_reader_reset_error_handler, 0);
-
-  rb_define_method(cXMLReader, "relax_ng_validate", ruby_xml_reader_relax_ng_validate, 1);
+  rb_define_method(cXMLReader, "relax_ng_validate", rxml_reader_relax_ng_validate, 1);
 #if LIBXML_VERSION >= 20620
-  rb_define_method(cXMLReader, "schema_validate", ruby_xml_reader_schema_validate, 1);
+  rb_define_method(cXMLReader, "schema_validate", rxml_reader_schema_validate, 1);
 #endif
   
-  rb_define_method(cXMLReader, "node_type", ruby_xml_reader_node_type, 0);
-  rb_define_method(cXMLReader, "normalization", ruby_xml_reader_normalization, 0);
-  rb_define_method(cXMLReader, "attribute_count", ruby_xml_reader_attr_count, 0);
-  rb_define_method(cXMLReader, "name", ruby_xml_reader_name, 0);
-  rb_define_method(cXMLReader, "local_name", ruby_xml_reader_local_name, 0);
-  rb_define_method(cXMLReader, "encoding", ruby_xml_reader_encoding, 0);
-  rb_define_method(cXMLReader, "base_uri", ruby_xml_reader_base_uri, 0);
-  rb_define_method(cXMLReader, "namespace_uri", ruby_xml_reader_namespace_uri, 0);
-  rb_define_method(cXMLReader, "xml_lang", ruby_xml_reader_xml_lang, 0);
-  rb_define_method(cXMLReader, "xml_version", ruby_xml_reader_xml_version, 0);
-  rb_define_method(cXMLReader, "prefix", ruby_xml_reader_prefix, 0);
-  rb_define_method(cXMLReader, "depth", ruby_xml_reader_depth, 0);
-  rb_define_method(cXMLReader, "quote_char", ruby_xml_reader_quote_char, 0); 
-  rb_define_method(cXMLReader, "standalone", ruby_xml_reader_standalone, 0);
+  rb_define_method(cXMLReader, "node_type", rxml_reader_node_type, 0);
+  rb_define_method(cXMLReader, "normalization", rxml_reader_normalization, 0);
+  rb_define_method(cXMLReader, "attribute_count", rxml_reader_attr_count, 0);
+  rb_define_method(cXMLReader, "name", rxml_reader_name, 0);
+  rb_define_method(cXMLReader, "local_name", rxml_reader_local_name, 0);
+  rb_define_method(cXMLReader, "encoding", rxml_reader_encoding, 0);
+  rb_define_method(cXMLReader, "base_uri", rxml_reader_base_uri, 0);
+  rb_define_method(cXMLReader, "namespace_uri", rxml_reader_namespace_uri, 0);
+  rb_define_method(cXMLReader, "xml_lang", rxml_reader_xml_lang, 0);
+  rb_define_method(cXMLReader, "xml_version", rxml_reader_xml_version, 0);
+  rb_define_method(cXMLReader, "prefix", rxml_reader_prefix, 0);
+  rb_define_method(cXMLReader, "depth", rxml_reader_depth, 0);
+  rb_define_method(cXMLReader, "quote_char", rxml_reader_quote_char, 0); 
+  rb_define_method(cXMLReader, "standalone", rxml_reader_standalone, 0);
   
-  rb_define_method(cXMLReader, "has_attributes?", ruby_xml_reader_has_attributes, 0);
-  rb_define_method(cXMLReader, "[]", ruby_xml_reader_attribute, 1);
-  rb_define_method(cXMLReader, "has_value?", ruby_xml_reader_has_value, 0);
-  rb_define_method(cXMLReader, "value", ruby_xml_reader_value, 0);
+  rb_define_method(cXMLReader, "has_attributes?", rxml_reader_has_attributes, 0);
+  rb_define_method(cXMLReader, "[]", rxml_reader_attribute, 1);
+  rb_define_method(cXMLReader, "has_value?", rxml_reader_has_value, 0);
+  rb_define_method(cXMLReader, "value", rxml_reader_value, 0);
 
-  rb_define_method(cXMLReader, "lookup_namespace", ruby_xml_reader_lookup_namespace, 1);
-  rb_define_method(cXMLReader, "expand", ruby_xml_reader_expand, 0);
+  rb_define_method(cXMLReader, "lookup_namespace", rxml_reader_lookup_namespace, 1);
+  rb_define_method(cXMLReader, "expand", rxml_reader_expand, 0);
 
 #if LIBXML_VERSION >= 20618
-  rb_define_method(cXMLReader, "byte_consumed", ruby_xml_reader_byte_consumed, 0);
+  rb_define_method(cXMLReader, "byte_consumed", rxml_reader_byte_consumed, 0);
 #endif
 #if LIBXML_VERSION >= 20617
-  rb_define_method(cXMLReader, "column_number", ruby_xml_reader_column_number, 0);
-  rb_define_method(cXMLReader, "line_number", ruby_xml_reader_line_number, 0);
+  rb_define_method(cXMLReader, "column_number", rxml_reader_column_number, 0);
+  rb_define_method(cXMLReader, "line_number", rxml_reader_line_number, 0);
 #endif
-  rb_define_method(cXMLReader, "default?", ruby_xml_reader_default, 0);
-  rb_define_method(cXMLReader, "empty_element?", ruby_xml_reader_empty_element, 0);
-  rb_define_method(cXMLReader, "namespace_declaration?", ruby_xml_reader_namespace_declaration, 0);
-  rb_define_method(cXMLReader, "valid?", ruby_xml_reader_valid, 0);
+  rb_define_method(cXMLReader, "default?", rxml_reader_default, 0);
+  rb_define_method(cXMLReader, "empty_element?", rxml_reader_empty_element, 0);
+  rb_define_method(cXMLReader, "namespace_declaration?", rxml_reader_namespace_declaration, 0);
+  rb_define_method(cXMLReader, "valid?", rxml_reader_valid, 0);
 
   rb_define_const(cXMLReader, "LOADDTD", INT2FIX(XML_PARSER_LOADDTD));
   rb_define_const(cXMLReader, "DEFAULTATTRS", INT2FIX(XML_PARSER_DEFAULTATTRS));
@@ -936,6 +900,7 @@ ruby_init_xml_reader(void)
   rb_define_const(cXMLReader, "TYPE_END_ENTITY", INT2FIX(XML_READER_TYPE_END_ENTITY));
   rb_define_const(cXMLReader, "TYPE_XML_DECLARATION", INT2FIX(XML_READER_TYPE_XML_DECLARATION));
 
+  /* Read states */
   rb_define_const(cXMLReader, "MODE_INITIAL", INT2FIX(XML_TEXTREADER_MODE_INITIAL));
   rb_define_const(cXMLReader, "MODE_INTERACTIVE", INT2FIX(XML_TEXTREADER_MODE_INTERACTIVE));
   rb_define_const(cXMLReader, "MODE_ERROR", INT2FIX(XML_TEXTREADER_MODE_ERROR));
