@@ -1,7 +1,7 @@
 /* $Id$ */
 
-#ifndef __RUBY_XML_STATE__
-#define __RUBY_XML_STATE__
+#ifndef __rxml_STATE__
+#define __rxml_STATE__
 
 extern VALUE cXMLState;
 extern VALUE LIBXML_STATE;
