@@ -1,12 +1,17 @@
-/* $Id: ruby_xml_xinclude.c 461 2008-07-15 21:35:56Z cfis $ */
-
-/* Please see the LICENSE file for copyright and distribution information */
+/* $Id: rxml_xinclude.c 566 2008-11-18 06:53:36Z cfis $ */
 
 #include "ruby_libxml.h"
 #include "ruby_xml_xinclude.h"
 
 VALUE cXMLXInclude;
-VALUE eXMLXIncludeError;
+
+/*
+* Document-class: LibXML::XML::XInclude
+*
+* The ruby bindings do not currently expose libxml's 
+* XInclude fuctionality.
+*/
+
 
 // Rdoc needs to know 
 #ifdef RDOC_NEVER_DEFINED
@@ -17,5 +22,4 @@ VALUE eXMLXIncludeError;
 void
 ruby_init_xml_xinclude(void) {
   cXMLXInclude = rb_define_class_under(mXML, "XInclude", rb_cObject);
-  eXMLXIncludeError = rb_define_class_under(cXMLXInclude, "Error", rb_eRuntimeError);
 }
