@@ -1,9 +1,9 @@
-/* $Id: ruby_xml_xinclude.h 39 2006-02-21 20:40:16Z roscopeco $ */
+/* $Id: ruby_xml_xinclude.h 612 2008-11-21 08:01:29Z cfis $ */
 
 /* Please see the LICENSE file for copyright and distribution information */
 
-#ifndef __RUBY_XML_XINCLUDE__
-#define __RUBY_XML_XINCLUDE__
+#ifndef __rxml_XINCLUDE__
+#define __rxml_XINCLUDE__
 
 extern VALUE cXMLXInclude;
 extern VALUE eXMLXIncludeError;
