@@ -1,4 +1,4 @@
-# $Id: libxml.rb 509 2008-07-22 02:11:00Z cfis $ 
+# $Id: libxml.rb 561 2008-11-18 04:17:29Z cfis $ 
 # Please see the LICENSE file for copyright and distribution information 
 
 # If running on Windows, then add the current directory to the PATH
@@ -12,14 +12,20 @@ end
 require 'libxml_ruby'
 
 # Load Ruby supporting code.
+require 'libxml/error'
 require 'libxml/parser'
 require 'libxml/parser_options'
+require 'libxml/parser_context'
 require 'libxml/document'
 require 'libxml/node'
-require 'libxml/node_set'
 require 'libxml/attributes'
 require 'libxml/attr'
 require 'libxml/tree'
+require 'libxml/reader'
+require 'libxml/html_parser'
+require 'libxml/sax_parser'
+require 'libxml/sax_callbacks'
+require 'libxml/xpath_object'
 
 # Deprecated
 require 'libxml/properties'
