@@ -1,8 +1,9 @@
-require 'tc_well_formed'
+require 'tc_xml'
 require 'tc_attributes'
 require 'tc_document'
 require 'tc_document_write'
 require 'tc_dtd'
+require 'tc_input'
 require 'tc_html_parser'
 require 'tc_node'
 require 'tc_node_attr'
@@ -10,10 +11,9 @@ require 'tc_node_cdata'
 require 'tc_node_comment'
 require 'tc_node_copy'
 require 'tc_node_edit'
-require 'tc_node_set'
-require 'tc_node_set2'
 require 'tc_node_text'
 require 'tc_node_xlink'
+require 'tc_ns'
 require 'tc_parser'
 require 'tc_parser_context'
 require 'tc_reader'
@@ -21,8 +21,10 @@ require 'tc_relaxng'
 require 'tc_sax_parser'
 require 'tc_schema'
 require 'tc_traversal'
+require 'tc_well_formed'
 require 'tc_xinclude'
 require 'tc_xpath'
+require 'tc_xpath_expression'
 require 'tc_xpointer'
 
 # Compatibility
